﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright company="Microsoft Corp." file="DeleteDeploymentOperation.cs">
//   All rights reserved. See License.txt in the project root for license information.
// </copyright>
// 
// --------------------------------------------------------------------------------------------------------------------
namespace CloudBlitz.DeploymentManager.DeploymentOperations
{
    using System;
    using System.Net;

    using AzureManagement.AzureDataContracts;

    using CloudBlitz.DeploymentManager.DeploymentWorkflow;

    public class DeleteDeploymentOperation : DeploymentOperationBase
    {
        private readonly IAzureServiceClientsFactory azureServiceClientsFactory;

        public DeleteDeploymentOperation(IAzureServiceClientsFactory azureServiceClientsFactory) : base(1)
        {
            this.azureServiceClientsFactory = azureServiceClientsFactory;
        }

        protected override OperationResult ExecuteOverride(DeploymentContext context)
        {
            var azureServiceManagementClient = this.azureServiceClientsFactory.CreateServiceManagementClient(context.AzureCredentials);

            try
            {
                this.RaiseProgressChangedEvent(0, string.Format("Deleting production deployment from hosted service \"{0}\"", context.HostedServiceName));

                var operationId = azureServiceManagementClient.DeleteDeployment(context.HostedServiceName, DeploymentSlots.Production).Result;

                this.RaiseProgressChangedEvent(50, string.Format("Production deployment is being deleted from hosted service \"{0}\", waiting for the operation to complete", context.HostedServiceName));
                var operation = azureServiceManagementClient.WaitOperationForCompletion(operationId, TimeSpan.MaxValue);

                if (operation.Status != OperationStatuses.Succeeded)
                {
                    return new OperationResult(DeploymentOperationStatus.Failed, operation.Error.Message);
                }
            }
            catch (AggregateException e)
            {
                if (e.InnerException is WebException)
                {
                    var webException = e.InnerException as WebException;
                    var httpWebResponse = webException.Response as HttpWebResponse;
                    if (httpWebResponse != null && httpWebResponse.StatusCode == HttpStatusCode.NotFound)
                    {
                        this.RaiseProgressChangedEvent(100, string.Format("Production deployment for hosted service \"{0}\" doesn't exist", context.HostedServiceName));
                        return OperationResult.Success;
                    }
                }

                return ExceptionHelper.HandleException(e);
            }
            catch (Exception e)
            {
                return ExceptionHelper.HandleException(e);
            }

            this.RaiseProgressChangedEvent(100, string.Format("Production deployment has been deleted from hosted service \"{0}\"", context.HostedServiceName));
            return OperationResult.Success;
        }
    }
}