﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright company="Microsoft Corp." file="DeleteSqlServerOperation.cs">
//   All rights reserved. See License.txt in the project root for license information.
// </copyright>
// 
// --------------------------------------------------------------------------------------------------------------------
namespace CloudBlitz.DeploymentManager.DeploymentOperations
{
    using System;
    using System.Net;

    using AzureManagement.AzureDataContracts;

    using CloudBlitz.DeploymentManager.DeploymentWorkflow;

    public class DeleteSqlServerOperation : DeploymentOperationBase
    {
        private readonly IAzureServiceClientsFactory azureServiceClientsFactory;

        public DeleteSqlServerOperation(IAzureServiceClientsFactory azureServiceClientsFactory) : base(1)
        {
            this.azureServiceClientsFactory = azureServiceClientsFactory;
        }

        protected override OperationResult ExecuteOverride(DeploymentContext context)
        {
            var serviceManagementClient = this.azureServiceClientsFactory.CreateServiceManagementClient(context.AzureCredentials);
            var sqlAzureManagementClient = this.azureServiceClientsFactory.CreateSqlAzureManagementClient(context.AzureCredentials);

            try
            {
                this.RaiseProgressChangedEvent(0, string.Format("Deleting SQL server \"{0}\"", context.SqlServerDetails.Name));
                if (!string.IsNullOrEmpty(context.SqlServerDetails.Name))
                {
                    var operationId = sqlAzureManagementClient.DeleteSqlAzureServer(context.SqlServerDetails.Name).Result;

                    this.RaiseProgressChangedEvent(50, string.Format("SQL server \"{0}\" is being deleted, waiting for the operation to complete", context.SqlServerDetails.Name));
                    var operation = serviceManagementClient.WaitOperationForCompletion(operationId, TimeSpan.MaxValue);

                    if (operation.Status == OperationStatuses.Failed)
                    {
                        return new OperationResult(DeploymentOperationStatus.Failed, operation.Error.Message);
                    }
                }
            }
            catch (AggregateException e)
            {
                if (e.InnerException is WebException)
                {
                    var webException = e.InnerException as WebException;
                    var httpWebResponse = webException.Response as HttpWebResponse;
                    if (httpWebResponse != null && httpWebResponse.StatusCode == HttpStatusCode.NotFound)
                    {
                        this.RaiseProgressChangedEvent(100, string.Format("SQL server \"{0}\" doesn't exist", context.SqlServerDetails.Name));
                        return OperationResult.Success;
                    }
                }

                return ExceptionHelper.HandleException(e);
            }
            catch (Exception e)
            {
                return ExceptionHelper.HandleException(e);
            }

            this.RaiseProgressChangedEvent(100, string.Format("SQL server \"{0}\" has been deleted", context.SqlServerDetails.Name));
            return OperationResult.Success;
        }
    }
}